import math
import pdb

from OpenGL import GL

pull_factor = 0.2
pull_distance = 5

def draw(ship1, ship2):
	GL.glBegin(GL.GL_LINES)
	GL.glVertex2f(*ship1.position)
	GL.glVertex2f(*ship2.position)
	GL.glEnd()

def pull(ship1, ship2, time_passed):
	distancex = ship1.position[0]-ship2.position[0]
	distancey = ship1.position[1]-ship2.position[1]
	distance = math.sqrt(distancex*distancex + distancey*distancey)
	force = pull_factor * max(0, distance-pull_distance) / distance
	ship1.velocity[0] -= distancex * force
	ship1.velocity[1] -= distancey * force
	ship2.velocity[0] += distancex * force
	ship2.velocity[1] += distancey * force

def pt_move(pt,diff):
	return (pt[0] + diff[0],pt[1] + diff[1])

def pt_rotate(pt,ang):
	rotated_pt_x = math.cos(ang)*pt[0] - math.sin(ang)*pt[1]
	rotated_pt_y = math.sin(ang)*pt[0] + math.cos(ang)*pt[1]
	return (rotated_pt_x, rotated_pt_y)

def cross(ship1,ship2,ship3):
	s1 = (ship1.last_position[0],ship1.last_position[1])
	s2 = (ship2.last_position[0],ship2.last_position[1])
	s3 = (ship3.last_position[0],ship3.last_position[1])
	s1t = (ship1.position[0],ship1.position[1])
	s2t = (ship2.position[0],ship2.position[1])
	s3t = (ship3.position[0],ship3.position[1])
	#
	# pdb.set_trace() enable this to trace :)
	
	# STEP 1: moving s1 to 0,0
	diff = (-s1[0],-s1[1])
	s1 = pt_move(s1,diff)
	s2 = pt_move(s2,diff)
	s3 = pt_move(s3,diff)
	diff = (-s1t[0],-s1t[1])
	s1t = pt_move(s1t,diff)
	s2t = pt_move(s2t,diff)
	s3t = pt_move(s3t,diff)
	#
	del diff
	
	#STEP 2: rotating s2 to the positive x axis
	# getting avg len of s2
	s2len = math.sqrt((s2[0]*s2[0] + s2[1]*s2[1]))
	s2lent = math.sqrt((s2t[0]*s2t[0] + s2t[1]*s2t[1]))
	s2len = (s2len + s2lent) / 2.0
	# getting turn angle (angle of s2)
	alpha = math.atan2(s2[1],s2[0])
	alphat = math.atan2(s2t[1],s2t[0])
	# rotating the s2
	s2 = (s2len,0)
	s2t = (s2len,0)
	# rotating s3 with the same angle
	s3 = pt_rotate(s3,-alpha)
	s3t = pt_rotate(s3t,-alphat)
	#
	del s2len,s2lent
	del alpha,alphat
	
	#STEP 3: intersecting s3-s3t with s1(0)-s2
	# line from s3 to s3t:
	if (s3t[0] - s3[0]) == 0: #same x -> alighed to the y axis
		return False
	#if s3-s3t is all above or below the x axis, there is no intersection with s1-s2
	if s3t[1]>0 and s3[1]>0:
		return False
	if s3t[1]<0 and s3[1]<0:
		return False
	#otherwise, there might be intersection - check WHERE s3-s3t intersects the x axis
	s3slope = (s3t[1] - s3[1])/(s3t[0] - s3[0])
	if s3slope  == 0: #aligned to the x axis
		return False
	col_x = s3[0] - s3[1]/s3slope
	#and see if that intersection with the x axis is where s1-s2 is
	if col_x < 0 or col_x > s2[0]:
		return False
	return True
